/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ident	"@(#)hpsl:hpsl.h	1.4.1.5"
#ident	"$Header: $"

/* 
 *  hpsl.h:
 *
 *  Header file shared by hotplug library functions and any user application
 *  that needs to access functions and data structures in hotplug library.
 */

#ifndef _HPSL_H
#define _HPSL_H

#if defined(__cplusplus)
extern "C" {
#endif

/***************************************/

#include <sys/hpci.h>      /* header file for Hpci data structures */

/***************************************/

#define HPSL_BUFSIZE        128

#define HPSL_FALSE          0
#define HPSL_TRUE           1

#define HPSL_CLEAR          0
#define HPSL_SET            1

/***************************************/

/* return codes of hotplug library functions */
#define HPSL_SUCCESS    1
#define HPSL_FAIL       2
#define HPSL_ERROR      3
#define HPCI_ERROR      4

/* 
 *  specific error type communicted to user application 
 *  from hotplug lib via hpsl_ERR global variable 
 */

#define hpsl_eINIT                 11
#define hpsl_eDEVOPEN              12
#define hpsl_eOPENDIR              13
#define hpsl_eDBOPEN               14
#define hpsl_eFOPEN                15
#define hpsl_ePOPEN                16
#define hpsl_eNOMEM                17
#define hpsl_eSTAT                 18
#define hpsl_eEXIST                19
#define hpsl_ePROTO                20
#define hpsl_eCOMPAT               21
#define hpsl_eAUTOCONF             22
#define hpsl_eNOCONT               23
#define hpsl_eMASK                 24
#define hpsl_eROMASK               25
#define hpsl_eFLAG                 26
#define hpsl_eSOCKETEMPTY          27
#define hpsl_eSOCKETPWR            28
#define hpsl_eSOCKETLOCK           29
#define hpsl_eSOCKETBUSY           30
#define hpsl_eINVALID              31
#define hpsl_eDRVLOAD              32

/***************************************/

/* 
 *  Event type returned by hpsl_check_async_events() 
 *  if rescan is recommended 
 */

#define EV_RESCAN             -1

/***************************************/

typedef enum _AsyncEvent {NO_CHANGE, NEW_EVENT} AsyncEvent_e;

/***************************************/

typedef struct _HpslContInfo      *HpslContInfoPtr_t;
typedef struct _HpslBusInfo       *HpslBusInfoPtr_t;
typedef struct _HpslSocketInfo    *HpslSocketInfoPtr_t;
typedef struct _PciSocketInfo     *PciSocketInfoPtr_t;
typedef struct _DevInfo           *DevInfoPtr_t;
typedef struct _PciDevInfo        *PciDevInfoPtr_t;
typedef struct _SuspendedDrvInfo  *SuspendedDrvInfoPtr_t;

/***************************************/

/* Data representation of a hotplug controller */
typedef struct _HpslContInfo
{
	unsigned int             contId;                           /* controller id */
	ushort                   hpcBusCnt;                        /* # of buses controlled by this controller */
	HpslBusInfoPtr_t         hpcBusList;                       /* pointer to buses */
} HpslContInfo_t;

/* Data representation of a bus connected to a hotplug controller */
typedef struct _HpslBusInfo
{
	unsigned int             contId;                           /* controller id */
	unsigned int             busId;                            /* bus id */
	unsigned char            busType;                          /* bus type */
	ushort                   maxBusSpeed;                      /* bus speed (max) */
	ushort                   presentBusSpeed;                  /* bus speed (current) */
	ushort                   busSocketCnt;                     /* # of sockets belonging to this bus */
	HpslSocketInfoPtr_t      busSocketList;                      /* pointer to sockets */
} HpslBusInfo_t;

/* Specific data representation of a socket connected to a PCI bus */
typedef struct _PciSocketInfo
{
	ushort                   slotNumber;                       /* pci slot number */
	ushort                   cardSpeed;                        /* board running speed */
	uchar_t                  memType;                          /* memory type */
} PciSocketInfo_t;

/* Data representation of a socket connected to a bus */
typedef struct _HpslSocketInfo
{
	unsigned int             contId;                           /* controller id */
	unsigned int             busId;                            /* bus id */
	char                     socketLabel[HPCI_MAX_SOCKET_LABEL_LEN];/* socket identifier label */
	unsigned char            busType;                          /* bus type of this socket */
	unsigned int             socketMask;                       /* supported state bits */
	unsigned int             socketROmask;                     /* read only - supported state bits */
	unsigned int             socketCurrentState;               /* current state bits */
	unsigned int             socketPreviousState;              /* previous state bits */
	union
	{
		PciSocketInfo_t        pcisocket_info_t;               /* pci bus specific socket info */
		char                   reserveBuf[HPSL_BUFSIZE];       /* filler */
	} uSocketInfo;
	AsyncEvent_e             socketEvent;                      /* async event flag */
	ushort                   devCnt;                           /* # of devices connected to this socket */
	DevInfoPtr_t             devList;                          /* pointer to devices */
} HpslSocketInfo_t;

/* Specific data representation of a device connected to a socket on a PCI bus */
typedef struct _PciDevInfo
{
	ushort                   busId;                            /* bus id */
	ushort                   devNumber;                        /* device number */
	uchar_t                  funcNumber;                       /* function number */
} PciDevInfo_t;

/* Data representation of a driver */
/* possible values for drvType*/
#define HPSL_UNKNOWN	2
#define HPSL_HBA	0
#define HPSL_NIC	1

typedef struct _HpslDrvInfo_t
{
	char	drvName[RM_MAXPARAMLEN];
	int	drvType;
}HpslDrvInfo_t;

/* Data representation of a device connected to a socket */
typedef struct _DevInfo
{
	rm_key_t                 rmKey;                            /* device resmgr key */
	HpslDrvInfo_t		 drvInfo;          /* driver information */
	union
	{
		PciDevInfo_t         pciDevInfo;                       /* pci bus specific device info */
		char                 reserveBuf[HPSL_BUFSIZE];         /* filler */
	} uDevInfo;
} DevInfo_t;

/* Data representation of a suspended driver instance */
typedef struct _SuspendedDrvInfo
{
	int                      suspId;                           /* suspended instance id */
	HpslDrvInfo_t		 drvInfo;
	HpslSocketInfo_t         socketInfo;                       /* socket to which the device belongs */
} SuspendedDrvInfo_t;

/***************************************/

/*
 *  Library functions - visible to the user application
 *  Naming convention - hpsl_XXX(xxx);
 */

int  hpsl_init_cont_info(HpslContInfo_t **, uint *);
int  hpsl_set_state(HpslSocketInfoPtr_t, uint, uint);
int  hpsl_bind_driver(HpslSocketInfoPtr_t, rm_key_t, char *);
int  hpsl_unbind_driver(HpslSocketInfoPtr_t, rm_key_t);
int  hpsl_suspend_io(HpslSocketInfoPtr_t, rm_key_t);
int  hpsl_resume_io(HpslSocketInfoPtr_t, rm_key_t, int);
int  hpsl_remove_driver(char *, int);
int  hpsl_get_suspended_drvlist(SuspendedDrvInfo_t **, uint *);
int  hpsl_check_async_events(int *);
int  hpsl_match_devdrv(rm_key_t, char *);
int  hpsl_check_drv_capability(char *, int);
void hpsl_cleanup(void);
int  hpsl_find_classcode(rm_key_t, uint *);
int  hpsl_find_drvnames(rm_key_t, HpslDrvInfo_t **, uint *);
int  hpsl_is_boot_hba(rm_key_t);
int  hpsl_isin_suspendlist(int);

/***************************************/

#if defined(__cplusplus)
}
#endif
#endif /* _HPSL_H */
